#!/usr/bin/perl -w
#
# usage: convert.pl inputfile.txt
#
#  inputfile.txt may be in Q&A format or tab-delimited format.
#  convert.pl will write out inputfile.xml to be used with 
#  supermemox
#
#
#   Robert W. Brewer <rbrewer@op.net>
#   Copyright (C) 2000 Robert W. Brewer
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */ 
#

use strict;


my $unfiled_cat = "Unfiled";
my $unfiled_cat_int = 0xffff;



sub write_item_record
{
    # write out a single item record
    # input: output filehandle
    # input: array of item field values and category name

    my $outfile = shift;
    my @item = @_;

    print $outfile "<knowledgenugget>\n";
    print $outfile "<question>$item[0]</question>\n";
    print $outfile "<answer>$item[1]";

    my $i;
    for $i (2..5)
      {
	if ($item[$i] ne "" )
	  {
	    print $outfile "\n$item[$i]";
	  }
      }
    print $outfile "</answer>\n";
    print $outfile "<desired_retention>0.9</desired_retention>\n";
    print $outfile "</knowledgenugget>\n";
}



# parse args
my $infile = shift;

#open input file
open INPUT, "<$infile" or die "Can't open input file $infile\n";


my $cat_counter = 0;

my %category;
$category{$unfiled_cat} = $unfiled_cat_int;
my %cat_items;
$cat_counter++;

my @all_items = ();
my @item = ();
my $item_counter = 0;
my $category;
my $qa_cat = "";
my @qa_item = ();
my $last_field = -1;

sub add_item
{
  # add an item to the data structures
  # input: item array in format from tab-delimited input


  push @all_items, @_; # add to list of all items

  push @{ $cat_items{$_[6]} }, $item_counter; # add to category tracking

  $item_counter++;
  push @all_items, 0;  # tack length on here
    # (left in so data structure same as palm converter


  # update categories
  if (not $category{$_[6]})
    {
      $category{$_[6]} = $cat_counter - 1; # -1 accounts for
      # Unfiled category
      # so that user cats start
      # at 0
      $cat_counter++;
    }
}


sub finish_qa_item
  {
    # complete the last QA item
    # time to finish this record

    if (scalar(@qa_item) == 0)
      {
	return;
      }

    while ($#qa_item < 6) # initialize unspecified fields
      {
	push @qa_item, "";
      }

    if ($qa_cat eq "")
      {
	$qa_cat = $unfiled_cat;
      }

    $qa_item[6] = $qa_cat;

    add_item(@qa_item);

    @qa_item = ();
    $qa_cat = "";
    $last_field = -1;
  }


while (<INPUT>)
  {
    next if /^\//; # skip comment lines ("/" first character)
    next if /^\s$/; # skip blank lines

    chomp; # remove trailing newline
    s/\r$//;  # remove carriage return if present

    # FIXME need to handle line break substitutions correctly
    # for both tab-delimited and Q&A format.

    # FIXME need to handle two lines with same identifier in a row
    # to cause a line break in the resulting file for Q&A format.

    if (s/^([cCqQaA123456])://)  # must be a Q&A file, chop off up to ":"
      {
	my $field = $1;
	print "f: --$field--\n";

	if ($field eq "c" or $field eq "C")
	  {
	    finish_qa_item;

	    $qa_cat = $_;
	    print "cat: $qa_cat\n";

	    # remove white space at either side of category value
	    $qa_cat =~ s/^\s//;
	    $qa_cat =~ s/\s$//;

	    $last_field = 0;
	  }
	elsif ($field eq "q" or $field eq "Q" or $field eq "1")
	  {
	    if ($last_field >= 1)
	      {
		finish_qa_item;
	      }
	
	    print "1: $_\n";
	    push @qa_item, $_;
	    $last_field = 1;
	  }
	elsif ($field eq "a" or $field eq "A" or $field eq "2")
	  {
	    if ($last_field >= 2)
	      {
		finish_qa_item;
	      }
	
	    while (scalar(@qa_item) < 2-1)
	      {
		push @qa_item, "";
	      }
	    push @qa_item, $_;
	    $last_field = 2;
	  }
	elsif ($field eq "3")
	  {
	    if ($last_field >= 3)
	      {
		finish_qa_item;
	      }
	
	    while (scalar(@qa_item) < 3-1)
	      {
		push @qa_item, ""; 
	      }
	    push @qa_item, $_;
	    $last_field = 3;
	  }
	elsif ($field eq "4")
	  {
	    if ($last_field >= 4)
	      {
		finish_qa_item;
	      }
	
	    while (scalar(@qa_item) < 4-1)
	      {
		push @qa_item, ""; 
	      }

	    push @qa_item, $_;
	    $last_field = 4;
	  }
	elsif ($field eq "5")
	  {
	    if ($last_field >= 5)
	      {
		finish_qa_item;
	      }
	
	    while (scalar(@qa_item) < 5-1)
	      {
		push @qa_item, ""; 
	      }

	    push @qa_item, ("", "", "", ""); # field 1, 2, 3, 4
	    push @qa_item, $_;
	    $last_field = 5;
	  }
	elsif ($field eq "6")
	  {
	    if ($last_field >= 6)
	      {
		finish_qa_item;
	      }
	
	    while (scalar(@qa_item) < 6-1)
	      {
		push @qa_item, ""; 
	      }

	    push @qa_item, $_;
	    $last_field = 6;
	  }
	else
	  {
	    die "Error in code.";
	  }
      }
    else
      {
	finish_qa_item;

        @item = split /\t/; # break into tab-delimited fields
	
	while ($#item < 6) # initialize unspecified fields
	  {
	    push @item, "";
	  }
	
	# remove white space at either side of category name
	$item[6] =~ s/^\s//; 
	$item[6] =~ s/\s$//;

	if ($item[6] eq "") # give it the unfiled category if none specified
	  {
	    $item[6] = $unfiled_cat;
	  }

	add_item(@item);
      }

  }


# need one last finish to get the last record added
finish_qa_item;

#while (@all_items)
#  {
#    my $s = shift @all_items;
#    print "--$s--\n";
#  }


# open output file
my $outfile;
($outfile = $infile) =~ s/\.[^.]*$//;
$outfile .= ".xml";

open OUTPUT, ">$outfile" or die "Can't open output file $outfile\n";

# header stuff
print OUTPUT "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
print OUTPUT "<supermemox createdby='convert.pl' fileformat='0'>\n";
print OUTPUT "<knowledgecollection>\n";
print OUTPUT "<name>root</name>\n";


# do all user-defined categories in order
my $key;
foreach $key (sort { $category{$a} <=> $category{$b} }
	      keys %category)
{
  # for Unfiled category items, we put them in the root
  # collection.  Items with an explicit category get 
  # their own subcollection.  supermemo for the palm
  # only has one level of heirarchy
  if ($key ne $unfiled_cat)
    {
      print OUTPUT "<knowledgecollection>\n";
      print OUTPUT "<name>$key</name>\n";
    }

  my $offset = 0;
  while ($offset < scalar(@all_items))
    {
      if ($all_items[$offset*8 + 6] eq $key)
	{
	  # item is in the category we are currently outputting
	  write_item_record(\*OUTPUT, splice(@all_items, 0, 8));

	  # splice will remove items from the array, so no need 
	  # to increment offset
	}
      else
	{
	  $offset += 8;
	}
    }

  if ($key ne $unfiled_cat)
    {
      print OUTPUT "</knowledgecollection>\n";
    }
}	


# footer stuff
print OUTPUT "</knowledgecollection>\n";
print OUTPUT "</supermemox>\n";

close OUTPUT;



